#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "sid.h"
#include <sys/types.h>
#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_nodeid.h"

ZEND_DECLARE_MODULE_GLOBALS(nodeid)

#ifdef ZTS
	int nodeid_globals_id;
#endif

	/* True global resources - no need for thread safety here */
	static int le_nodeid;

	ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_nodeid, 0, 0, IS_ARRAY, 0)
	ZEND_END_ARG_INFO()
	ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_serverid, 0, 0, IS_STRING, 0)
ZEND_END_ARG_INFO()

	/* {{{ PHP_INI
	*/

PHP_INI_BEGIN()
	STD_PHP_INI_ENTRY("nodeid.server_id","1", PHP_INI_ALL, OnUpdateLong, nodeid_server_id, zend_nodeid_globals, nodeid_globals)
	//STD_PHP_INI_ENTRY("nodeid.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_nodeid_globals, nodeid_globals)
PHP_INI_END()

	/* }}} */

	/* {{{ php proto nodeid(?int $timestamp, ?int $seed): array
	*/
PHP_FUNCTION(nodeid)
{
	zend_long ts_int;
	zend_long wid;

	zend_parse_parameters(ZEND_NUM_ARGS(), "|ll", &ts_int, &wid);

	long datacenterId = NODEID_G(nodeid_server_id);

	set_datacenterId(datacenterId);
	uint64_t unquie;
	uint64_t convertedValue;
	long seed;

	if (ZEND_NUM_ARGS() == 2) {
		convertedValue = (uint64_t)ts_int;
		seed = (long)wid;
	} else {
		convertedValue = timeGen();
		seed = get_pid();
	}

	set_workid(seed);
	unquie = get_twitter_id(convertedValue);

	array_init(return_value);
	add_assoc_long(return_value, "serverid", datacenterId);
	add_assoc_long(return_value, "timestamp", convertedValue);
	add_assoc_long(return_value, "seed", seed);
	add_assoc_long(return_value, "node_id", unquie);
}
/* }}} */

/* {{{ php proto nodeid_serverid() : int
*/
PHP_FUNCTION(nodeid_serverid)
{
	if (zend_parse_parameters_none() == FAILURE)
	{
		return;
	}
	long datacenterId = NODEID_G(nodeid_server_id);
	RETURN_LONG(datacenterId);
}
/* }}} */

/* {{{ php_nodeid_init_globals
*/
static void php_nodeid_init_globals(zend_nodeid_globals *nodeid_globals)
{
	nodeid_globals->nodeid_server_id = 1;
	nodeid_globals->global_string = NULL;
}
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
*/
PHP_MINIT_FUNCTION(nodeid)
{
#ifdef ZTS
	ts_allocate_id(&nodeid_globals_id, sizeof(zend_nodeid_globals), 
			(ts_allocate_ctor) php_nodeid_init_globals, NULL);
#else
	php_nodeid_init_globals(&nodeid_globals);
#endif

	if (NODEID_G(nodeid_server_id) < 0) {
		php_error_docref(NULL, E_WARNING, "nodeid.server_id must greater than 0");
		return FAILURE;
	}
	if (NODEID_G(nodeid_server_id) > 0x3FF) {
		php_error_docref(NULL, E_WARNING, "nodeid.server_id must less than %d", 0x3FF);
		return FAILURE;
	}
	REGISTER_INI_ENTRIES();

	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
*/
PHP_MSHUTDOWN_FUNCTION(nodeid)
{
	UNREGISTER_INI_ENTRIES();
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
*/
PHP_RINIT_FUNCTION(nodeid)
{
#if defined(ZTS) && defined(COMPILE_DL_NODEID)
	ZEND_TSRMLS_CACHE_UPDATE();
#endif
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
*/
PHP_RSHUTDOWN_FUNCTION(nodeid)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
*/
PHP_MINFO_FUNCTION(nodeid)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "NodeID support", "enabled");
	php_info_print_table_row(2, "author", "HSK Consulting");
	php_info_print_table_row(2, "version", PHP_NODEID_VERSION);
	php_info_print_table_end();
	DISPLAY_INI_ENTRIES();
}
/* }}} */

/* {{{ nodeid_functions[]
 *
 * Every user visible function must have an entry in nodeid_functions[].
 */
const zend_function_entry nodeid_functions[] = {
	PHP_FE(nodeid, arginfo_nodeid)
		PHP_FE(nodeid_serverid, arginfo_serverid)
		PHP_FE_END      /* Must be the last line in nodeid_functions[] */
};
/* }}} */

/* {{{ nodeid_module_entry
*/
zend_module_entry nodeid_module_entry = {
	STANDARD_MODULE_HEADER,
	"nodeid",
	nodeid_functions,
	PHP_MINIT(nodeid),
	PHP_MSHUTDOWN(nodeid),
	PHP_RINIT(nodeid),              /* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(nodeid),  /* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(nodeid),
	PHP_NODEID_VERSION,
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_NODEID
ZEND_GET_MODULE(nodeid)
#endif

	/*
	 * Local variables:
	 * tab-width: 4
	 * c-basic-offset: 4
	 * End:
	 * vim600: noet sw=4 ts=4 fdm=marker
	 * vim<600: noet sw=4 ts=4
	 */
