#!/bin/bash

# Colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
NC='\033[0m' # No Color

# Function to print status messages
print_status() {
	echo -e "${GREEN}[*] $1${NC}"
}

print_warning() {
	echo -e "${YELLOW}[!] $1${NC}"
}

print_error() {
	echo -e "${RED}[!] $1${NC}"
	exit 1
}

# Check command line arguments
if [ $# -lt 2 ] || [ $# -gt 4 ]; then
	print_error "Usage: $0 <php-version> [build-dir] [nts-final-dir] [zts-final-dir] \nExample: $0 8.4 /tmp/phpbuild /mnt/c/Users/henry/Onedrive/Skrivebord\nts /mnt/c/Users/henry/Onedrive/Skrivebord\zts"
fi

# Configuration
PHP_VERSION="$1"
BUILD_BASE="${2:-/tmp/phpbuild}"
FINAL_DIR_NTS="$3"
FINAL_DIR_ZTS="$4"

# Function to detect extension name from config.m4 or config.w32
detect_extension_name() {
	local ext_name=""

	if [ -f "config.m4" ]; then
		ext_name=$(grep -E "PHP_(NEW|ADD)_EXTENSION\(" config.m4 | head -1 | sed -E 's/.*PHP_(NEW|ADD)_EXTENSION\(([^,]*).*/\2/' | tr -d '[:space:]')
	fi

	if [ -z "$ext_name" ] && [ -f "config.w32" ]; then
		ext_name=$(grep -E "ADD_EXTENSION\(" config.w32 | head -1 | sed -E 's/.*ADD_EXTENSION\(([^,]*).*/\1/' | tr -d '[space:]')
	fi

	if [ -z "$ext_name" ]; then
		local c_files=(*.c)
		if [ -f "${c_files[0]}" ]; then
			ext_name=$(basename "${c_files[0]}" .c)
			print_warning "Extension name detected from .c file: ${ext_name}"
		else
			print_error "Could not detect extension name. Please check your source files."
		fi
	fi

	echo "$ext_name"
}

EXTENSION_NAME=$(detect_extension_name)

print_status "Building NTS version and copying to '${FINAL_DIR_NTS}' ... "
./secuno_ext_build_nts.sh "${PHP_VERSION}"
cp "modules/${EXTENSION_NAME}.so" "${FINAL_DIR_NTS}/" || print_error "Failed to copy NTS version"

print_status "Building ZTS version and copying to '${FINAL_DIR_ZTS}' ... "
./secuno_ext_build_zts.sh "${PHP_VERSION}" "${BUILD_BASE}"
cp "modules/${EXTENSION_NAME}.so" "${FINAL_DIR_ZTS}/" || print_error "Failed to copy ZTS version"

print_status "PHP Version: ${PHP_VERSION}"
print_status "NTS Final Destination: ${FINAL_DIR_NTS}/${EXTENSION_NAME}.so"
print_status "ZTS Final Destination: ${FINAL_DIR_ZTS}/${EXTENSION_NAME}.so"

